//-----------------------------------------------
// AiboBiff
// 2002/11/28  Kazuto Sato
//-----------------------------------------------
#include <string.h>
#include <OPENR/OSyslog.h>
#include <OPENR/OPENRAPI.h>
#include <OPENR/core_macro.h>
#include <ant.h>
#include <EndpointTypes.h>
#include <TCPEndpointMsg.h>
#include "AiboBiff.h"
#include "entry.h"

AiboBiff::AiboBiff ()
{
    // POP3СIPɥ쥹
    strcpy(serverip, "202.225.88.136");
    // POP3СΥݡֹ
    serverport = 110;
    // 
    strcpy(username, "kazuto");
    // ѥ
    strcpy(password, "*******");
}

OStatus
AiboBiff::DoInit(const OSystemEvent& event)
{
    OSYSDEBUG(("AiboBiff::DoInit()\n"));
    NEW_ALL_SUBJECT_AND_OBSERVER;
    REGISTER_ALL_ENTRY;
    SET_ALL_READY_AND_NOTIFY_ENTRY;
    return oSUCCESS;
}

OStatus
AiboBiff::DoStart(const OSystemEvent& event)
{
    OSYSDEBUG(("AiboBiff::DoStart()\n"));
    ENABLE_ALL_SUBJECT;
    ASSERT_READY_TO_ALL_OBSERVER;

    ipstackRef = antStackRef("IPStack");

    for (int index = 0; index < AIBOBIFF_CONNECTION_MAX; index++) {
        OStatus result = InitTCPConnection(index);
        if (result != oSUCCESS) return oFAIL;
    }

    Connect(0);

    return oSUCCESS;
}    

OStatus
AiboBiff::DoStop(const OSystemEvent& event)
{
    OSYSDEBUG(("AiboBiff::DoStop()\n"));
    DISABLE_ALL_SUBJECT;
    DEASSERT_READY_TO_ALL_OBSERVER;
    return oSUCCESS;
}

OStatus
AiboBiff::DoDestroy(const OSystemEvent& event)
{
    DELETE_ALL_SUBJECT_AND_OBSERVER;
    return oSUCCESS;
}

OStatus
AiboBiff::Connect(int index)
{
    OSYSDEBUG(("AiboBiff::Connect()\n"));

    if (connection[index].state != CONNECTION_CLOSED) return oFAIL;

    OSYSPRINT(("serverip is %s\n", serverip));
    TCPEndpointConnectMsg connectMsg(connection[index].endpoint,
                                     IP_ADDR_ANY, IP_PORT_ANY,
                                     serverip, serverport);
    connectMsg.continuation = (void*)index;

    connectMsg.Send(ipstackRef, myOID_,
                    Extra_Entry[entryConnectCont], sizeof(connectMsg));
    
    connection[index].state = CONNECTION_CONNECTING;

    return oSUCCESS;
}

void
AiboBiff::ConnectCont(void* msg)
{
    OSYSDEBUG(("AiboBiff::ConnectCont()\n"));

    TCPEndpointConnectMsg* connectMsg = (TCPEndpointConnectMsg*)msg;
    int index = (int)connectMsg->continuation;

    if (connectMsg->error != TCP_SUCCESS) {
        OSYSLOG1((osyslogERROR, "%s : %s %d",
                  "AiboBiff::ConnectCont()",
                  "FAILED. connectMsg->error", connectMsg->error));
        connection[index].state = CONNECTION_CLOSED;
        return;
    }

    connection[index].state = CONNECTION_CONNECTED;

    // 
    pop3state = POP3_CONNECTED;
    SetReceiveData(index);
    Receive(index);
}

OStatus
AiboBiff::Send(int index)
{
    OSYSDEBUG(("AiboBiff::Send()\n"));

    if (connection[index].sendSize == 0 ||
        connection[index].state != CONNECTION_CONNECTED) return oFAIL;

    TCPEndpointSendMsg sendMsg(connection[index].endpoint,
                               connection[index].sendData,
                               connection[index].sendSize);
    sendMsg.continuation = (void*)index;

    sendMsg.Send(ipstackRef, myOID_,
                 Extra_Entry[entrySendCont],
                 sizeof(TCPEndpointSendMsg));

    connection[index].state = CONNECTION_SENDING;
    connection[index].sendSize = 0;
    return oSUCCESS;
}

void
AiboBiff::SendCont(void* msg)
{
    OSYSDEBUG(("AiboBiff::SendCont()\n"));

    TCPEndpointSendMsg* sendMsg = (TCPEndpointSendMsg*)msg;
    int index = (int)(sendMsg->continuation);

    if (sendMsg->error != TCP_SUCCESS) {
        OSYSLOG1((osyslogERROR, "%s : %s %d",
                  "AiboBiff::SendCont()",
                  "FAILED. sendMsg->error", sendMsg->error));
        Close(index);
        return;
    }

    OSYSPRINT(("sendData : %s", connection[index].sendData));
    
    connection[index].state = CONNECTION_CONNECTED;

    SetReceiveData(index);
    Receive(index);
}

OStatus
AiboBiff::Receive(int index)
{
    OSYSDEBUG(("AiboBiff::Receive()\n"));

    if (connection[index].state != CONNECTION_CONNECTED &&
        connection[index].state != CONNECTION_SENDING) return oFAIL;

    TCPEndpointReceiveMsg receiveMsg(connection[index].endpoint,
                                     connection[index].recvData,
                                     1, connection[index].recvSize);
    receiveMsg.continuation = (void*)index;

    receiveMsg.Send(ipstackRef, myOID_,
                    Extra_Entry[entryReceiveCont], sizeof(receiveMsg));

    return oSUCCESS;
}

void
AiboBiff::ReceiveCont(void* msg)
{
    OSYSDEBUG(("AiboBiff::ReceiveCont()\n"));

    TCPEndpointReceiveMsg* receiveMsg = (TCPEndpointReceiveMsg*)msg;
    int index = (int)(receiveMsg->continuation);

    if (receiveMsg->error != TCP_SUCCESS) {
        OSYSLOG1((osyslogERROR, "%s : %s %d",
                  "AiboBiff::ReceiveCont()",
                  "FAILED. receiveMsg->error", receiveMsg->error));
        Close(index);
        return;
    }
    
    connection[index].recvData[receiveMsg->sizeMax] = 0;
    OSYSPRINT(("recvData : %s", connection[index].recvData));
    if(strncmp((char*)connection[index].recvData, "+OK", 3) != 0)
    {
        OSYSPRINT(("* POP3 Error *\n"));
        Close(index);
        return;
    }
    
    if(pop3state == POP3_CONNECTED) pop3state = POP3_USER;
    else if(pop3state == POP3_USER) pop3state = POP3_PASS;
    else if(pop3state == POP3_PASS) pop3state = POP3_STAT;
    else if(pop3state == POP3_STAT)
    {
        // å뤫ɤå
        CheckStat((char*)connection[index].recvData);
        pop3state = POP3_QUIT;
    }
    else if(pop3state == POP3_QUIT) pop3state = POP3_CLOSE;
    
    if(pop3state != POP3_CLOSE)
    {
        SetSendData(index);
        Send(index);
    }
    else Close(index);
}

OStatus
AiboBiff::Close(int index)
{
    OSYSDEBUG(("AiboBiff::Close()\n"));

    if (connection[index].state == CONNECTION_CLOSED ||
        connection[index].state == CONNECTION_CLOSING) return oSUCCESS;

    TCPEndpointCloseMsg closeMsg(connection[index].endpoint);
    closeMsg.continuation = (void*)index;

    closeMsg.Send(ipstackRef, myOID_,
                  Extra_Entry[entryCloseCont], sizeof(closeMsg));

    connection[index].state = CONNECTION_CLOSING;

    return oSUCCESS;
}

void
AiboBiff::CloseCont(void* msg)
{
    OSYSDEBUG(("AiboBiff::CloseCont()\n"));

    TCPEndpointCloseMsg* closeMsg = (TCPEndpointCloseMsg*)msg;
    int index = (int)(closeMsg->continuation);

    connection[index].state = CONNECTION_CLOSED;
}

OStatus
AiboBiff::InitTCPConnection(int index)
{
    OSYSDEBUG(("AiboBiff::InitTCPConnection()\n"));

    //
    // Create endpoint
    //
    antEnvCreateEndpointMsg tcpCreateMsg(EndpointType_TCP,
                                         AIBOBIFF_BUFFER_SIZE * 2);

    tcpCreateMsg.Call(ipstackRef, sizeof(tcpCreateMsg));
    if (tcpCreateMsg.error != ANT_SUCCESS) {
        OSYSLOG1((osyslogERROR, "%s : %s[%d] antError %d",
                  "AiboBiff::Connect()",
                  "Can't create endpoint",
                  index, tcpCreateMsg.error));
        return oFAIL;
    }

    connection[index].endpoint = tcpCreateMsg.moduleRef;
    connection[index].state = CONNECTION_CLOSED;

    // 
    // Allocate send buffer
    //
    antEnvCreateSharedBufferMsg sendBufferMsg(AIBOBIFF_BUFFER_SIZE);

    sendBufferMsg.Call(ipstackRef, sizeof(sendBufferMsg));
    if (sendBufferMsg.error != ANT_SUCCESS) {
        OSYSLOG1((osyslogERROR, "%s : %s[%d] antError %d",
                  "AiboBiff::InitTCPConnection()",
                  "Can't allocate send buffer",
                  index, sendBufferMsg.error));
        return oFAIL;
    }

    connection[index].sendBuffer = sendBufferMsg.buffer;
    connection[index].sendBuffer.Map();
    connection[index].sendData
        = (byte*)(connection[index].sendBuffer.GetAddress());

    //
    // Allocate receive buffer
    //
    antEnvCreateSharedBufferMsg recvBufferMsg(AIBOBIFF_BUFFER_SIZE);

    recvBufferMsg.Call(ipstackRef, sizeof(recvBufferMsg));
    if (recvBufferMsg.error != ANT_SUCCESS) {
        OSYSLOG1((osyslogERROR, "%s : %s[%d] antError %d",
                  "AiboBiff::InitTCPConnection()",
                  "Can't allocate receive buffer",
                  index, recvBufferMsg.error));
        return oFAIL;
    }

    connection[index].recvBuffer = recvBufferMsg.buffer;
    connection[index].recvBuffer.Map();
    connection[index].recvData
        = (byte*)(connection[index].recvBuffer.GetAddress());

    return oSUCCESS;
}

void
AiboBiff::SetSendData(int index)
{
    char command[80];
    
    // pop3state˱ƥޥɤ
    if(pop3state == POP3_USER)
    {
        strcpy(command, "USER ");
        strcat(command, username);
        strcat(command, "\r\n");
    }
    else if(pop3state == POP3_PASS)
    {
        strcpy(command, "PASS ");
        strcat(command, password);
        strcat(command, "\r\n");
    }
    else if(pop3state == POP3_STAT)
    {
        strcpy(command, "STAT\r\n");
    }
    else if(pop3state == POP3_QUIT)
    {
        strcpy(command, "QUIT\r\n");
    }
    
    strcpy((char*)connection[index].sendData, command);
    connection[index].sendSize = strlen(command);
}

void
AiboBiff::SetReceiveData(int index)
{
    connection[index].recvSize = AIBOBIFF_BUFFER_SIZE;
}

//-----------------------------------------------
// STATޥɤΥ쥹ݥ󥹤å
//-----------------------------------------------
void AiboBiff::CheckStat(char* resp)
{
    OSYSDEBUG(("AiboBiff::CheckStat()\n"));
    
    char numstr[10];
    int i, num;
    char* p = resp;
    
    p += 3; while(isspace(*p)) p++;
    for(i = 0; isdigit(*p); i++) numstr[i] = *p++;
    numstr[i] = 0;
    num = atoi(numstr);
    if(num > 0)
    {
        // ֥BowSoundWAV
        OSYSPRINT(("Play Sound!\n"));
        subject[sbjSound]->SetData("Play\0", 5);
        subject[sbjSound]->NotifyObservers();
    }
    return;
}
