#ifndef BowSound_h_DEFINED
#define BowSound_h_DEFINED

#include <OPENR/OObject.h>
#include <OPENR/OSubject.h>
#include <OPENR/OObserver.h>
#include "WAV.h"
#include "def.h"

enum SoundState {
    SS_IDLE,
    SS_START,
    SS_PLAYING
};

static const char* const SPEAKER_LOCATOR = "PRM:/r1/c1/c2/c3/s1-Speaker:S1";
static const char* const BOW_SOUND = "BOW_SOUND";

class BowSound : public OObject {
public:
    BowSound();
    virtual ~BowSound() {}

    OSubject*  subject[numOfSubject];
    OObserver* observer[numOfObserver];

    virtual OStatus DoInit   (const OSystemEvent& event);
    virtual OStatus DoStart  (const OSystemEvent& event);
    virtual OStatus DoStop   (const OSystemEvent& event);
    virtual OStatus DoDestroy(const OSystemEvent& event);

    void NotifyCommand(const ONotifyEvent& event);
    void ReadyPlay(const OReadyEvent& event);

private:
    void      Play();
    void      OpenSpeaker();
    void      NewSoundVectorData();
    void      LoadWAV();
    void      SetPowerAndVolume();
    WAVError  CopyWAVTo(RCRegion* region);
    RCRegion* FindFreeRegion();
    bool      IsAllRegionFree();

    //
    // 8KHz 8bits MONO (8 * 1 * 1 * 32 = 256)
    //
    static const size_t SOUND_UNIT_SIZE  = 256; // bytes / 32ms
    static const size_t SOUND_NUM_BUFFER = 2;

    SoundState soundState;
    OPrimitiveID   speakerID;
    ODesignDataID  soundID;
    WAV            wav;
    RCRegion*      region[SOUND_NUM_BUFFER];
};

#endif // BowSound_h_DEFINED
