//-----------------------------------------------
// HttpTest
// 2003/01/27  Kazuto Sato
//-----------------------------------------------

#include <string.h>
#include <OPENR/OSyslog.h>
#include <OPENR/OPENRAPI.h>
#include <OPENR/core_macro.h>
#include <ant.h>
#include <EndpointTypes.h>
#include <TCPEndpointMsg.h>
#include "HttpTest.h"
#include "entry.h"

HttpTest::HttpTest ()
{
}

OStatus
HttpTest::DoInit(const OSystemEvent& event)
{
    OSYSDEBUG(("HttpTest::DoInit()\n"));
    NEW_ALL_SUBJECT_AND_OBSERVER;
    REGISTER_ALL_ENTRY;
    SET_ALL_READY_AND_NOTIFY_ENTRY;
    return oSUCCESS;
}

OStatus
HttpTest::DoStart(const OSystemEvent& event)
{
    OSYSDEBUG(("HttpTest::DoStart()\n"));
    ENABLE_ALL_SUBJECT;
    ASSERT_READY_TO_ALL_OBSERVER;

    ipstackRef = antStackRef("IPStack");

    for (int index = 0; index < HTTPTEST_CONNECTION_MAX; index++) {
        OStatus result = InitTCPConnection(index);
        if (result != oSUCCESS) return oFAIL;
    }

    for (int index = 0; index < HTTPTEST_CONNECTION_MAX; index++)
        Listen(index);

    return oSUCCESS;
}

OStatus
HttpTest::DoStop(const OSystemEvent& event)
{
    OSYSDEBUG(("HttpTest::DoStop()\n"));
    DISABLE_ALL_SUBJECT;
    DEASSERT_READY_TO_ALL_OBSERVER;
    return oSUCCESS;
}

OStatus
HttpTest::DoDestroy(const OSystemEvent& event)
{
    DISABLE_ALL_SUBJECT;
    return oSUCCESS;
}

OStatus
HttpTest::Listen(int index)
{
    OSYSDEBUG(("HttpTest::Listen()\n"));

    if (connection[index].state != CONNECTION_CLOSED) return oFAIL;

    //
    // Create endpoint
    //
    antEnvCreateEndpointMsg tcpCreateMsg(EndpointType_TCP,
                                         HTTPTEST_BUFFER_SIZE*2);
    tcpCreateMsg.Call(ipstackRef, sizeof(tcpCreateMsg));
    if (tcpCreateMsg.error != ANT_SUCCESS) {
        OSYSLOG1((osyslogERROR, "%s : %s[%d] antError %d",
                  "HttpTest::Listen()",
                  "Can't create endpoint",
                  index, tcpCreateMsg.error));
        return oFAIL;
    }
    connection[index].endpoint = tcpCreateMsg.moduleRef;

    //
    // Listen
    //
    TCPEndpointListenMsg listenMsg(connection[index].endpoint,
                                   IP_ADDR_ANY, HTTPTEST_PORT);
    listenMsg.continuation = (void*)index;

    listenMsg.Send(ipstackRef, myOID_,
                   Extra_Entry[entryListenCont], sizeof(listenMsg));
    
    connection[index].state = CONNECTION_LISTENING;

    return oSUCCESS;
}

void
HttpTest::ListenCont(void* msg)
{
    OSYSDEBUG(("HttpTest::ListenCont()\n"));

    TCPEndpointListenMsg* listenMsg = (TCPEndpointListenMsg*)msg;
    int index = (int)listenMsg->continuation;

    if (listenMsg->error != TCP_SUCCESS) {
        OSYSLOG1((osyslogERROR, "%s : %s %d",
                  "HttpTest::ListenCont()",
                  "FAILED. listenMsg->error", listenMsg->error));
        Close(index);
        return;
    }

    connection[index].state = CONNECTION_CONNECTED;

    SetReceiveData(index);
    Receive(index);
}

OStatus
HttpTest::Send(int index)
{
    OSYSDEBUG(("HttpTest::Send()\n"));

    if (connection[index].sendSize == 0 ||
        connection[index].state != CONNECTION_CONNECTED) return oFAIL;

    TCPEndpointSendMsg sendMsg(connection[index].endpoint,
                               connection[index].sendData,
                               connection[index].sendSize);
    sendMsg.continuation = (void*)index;

    sendMsg.Send(ipstackRef, myOID_,
                 Extra_Entry[entrySendCont],
                 sizeof(TCPEndpointSendMsg));

    connection[index].state = CONNECTION_SENDING;
    connection[index].sendSize = 0;
    return oSUCCESS;
}

void
HttpTest::SendCont(void* msg)
{
    OSYSDEBUG(("HttpTest::SendCont()\n"));

    TCPEndpointSendMsg* sendMsg = (TCPEndpointSendMsg*)msg;
    int index = (int)(sendMsg->continuation);

    if (sendMsg->error != TCP_SUCCESS) {
        OSYSLOG1((osyslogERROR, "%s : %s %d",
                  "HttpTest::SendCont()",
                  "FAILED. sendMsg->error", sendMsg->error));
        Close(index);
        return;
    }

    connection[index].state = CONNECTION_CONNECTED;
    Close(index);
}

OStatus
HttpTest::Receive(int index)
{
    OSYSDEBUG(("HttpTest::Receive()\n"));

    if (connection[index].state != CONNECTION_CONNECTED &&
        connection[index].state != CONNECTION_SENDING) return oFAIL;

    TCPEndpointReceiveMsg receiveMsg(connection[index].endpoint,
                                     connection[index].recvData,
                                     1, connection[index].recvSize);
    receiveMsg.continuation = (void*)index;

    receiveMsg.Send(ipstackRef, myOID_,
                    Extra_Entry[entryReceiveCont], sizeof(receiveMsg));

    return oSUCCESS;
}

void
HttpTest::ReceiveCont(void* msg)
{
    OSYSDEBUG(("HttpTest::ReceiveCont()\n"));

    TCPEndpointReceiveMsg* receiveMsg = (TCPEndpointReceiveMsg*)msg;
    int index = (int)(receiveMsg->continuation);

    if (receiveMsg->error != TCP_SUCCESS) {
        OSYSLOG1((osyslogERROR, "%s : %s %d",
                  "HttpTest::ReceiveCont()",
                  "FAILED. receiveMsg->error", receiveMsg->error));
        Close(index);
        return;
    }

    connection[index].recvData[receiveMsg->sizeMax] = 0;
    OSYSPRINT(("* Requested *\n"));
    printRecvData((char*)connection[index].recvData);
    
    SetSendData(index);
    Send(index);
}

OStatus
HttpTest::Close(int index)
{
    OSYSDEBUG(("HttpTest::Close()\n"));

    if (connection[index].state == CONNECTION_CLOSED ||
        connection[index].state == CONNECTION_CLOSING) return oFAIL;

    TCPEndpointCloseMsg closeMsg(connection[index].endpoint);
    closeMsg.continuation = (void*)index;

    closeMsg.Send(ipstackRef, myOID_,
                  Extra_Entry[entryCloseCont], sizeof(closeMsg));

    connection[index].state = CONNECTION_CLOSING;

    return oSUCCESS;
}

void
HttpTest::CloseCont(void* msg)
{
    OSYSDEBUG(("HttpTest::CloseCont()\n"));
    
    TCPEndpointCloseMsg* closeMsg = (TCPEndpointCloseMsg*)msg;
    int index = (int)(closeMsg->continuation);

    connection[index].state = CONNECTION_CLOSED;
    Listen(index);
}

OStatus
HttpTest::InitTCPConnection(int index)
{
    OSYSDEBUG(("HttpTest::InitTCPConnection()\n"));

    connection[index].state = CONNECTION_CLOSED;

    // 
    // Allocate send buffer
    //
    antEnvCreateSharedBufferMsg sendBufferMsg(HTTPTEST_BUFFER_SIZE);

    sendBufferMsg.Call(ipstackRef, sizeof(sendBufferMsg));
    if (sendBufferMsg.error != ANT_SUCCESS) {
        OSYSLOG1((osyslogERROR, "%s : %s[%d] antError %d",
                  "HttpTest::InitTCPConnection()",
                  "Can't allocate send buffer",
                  index, sendBufferMsg.error));
        return oFAIL;
    }

    connection[index].sendBuffer = sendBufferMsg.buffer;
    connection[index].sendBuffer.Map();
    connection[index].sendData
        = (byte*)(connection[index].sendBuffer.GetAddress());

    //
    // Allocate receive buffer
    //
    antEnvCreateSharedBufferMsg recvBufferMsg(HTTPTEST_BUFFER_SIZE);

    recvBufferMsg.Call(ipstackRef, sizeof(recvBufferMsg));
    if (recvBufferMsg.error != ANT_SUCCESS) {
        OSYSLOG1((osyslogERROR, "%s : %s[%d] antError %d",
                  "HttpTest::InitTCPConnection()",
                  "Can't allocate receive buffer",
                  index, recvBufferMsg.error));
        return oFAIL;
    }

    connection[index].recvBuffer = recvBufferMsg.buffer;
    connection[index].recvBuffer.Map();
    connection[index].recvData
        = (byte*)(connection[index].recvBuffer.GetAddress());

    return oSUCCESS;
}

void
HttpTest::SetSendData(int index)
{
    char* data = "HTTP/1.0 200 OK\r\n"
                 "Server: HttpTest\r\n"
                 "Content-Type: text/html\r\n"
                 "\r\n"
                 "<title>HttpTest</title>\r\n"
                 "<p>Hello, World\r\n";
    
    strcpy((char*)connection[index].sendData, data);
    connection[index].sendSize = strlen(data);
}

void
HttpTest::SetReceiveData(int index)
{
    connection[index].recvSize = HTTPTEST_BUFFER_SIZE;
}

//-----------------------------------------------
// o
//-----------------------------------------------
void HttpTest::printRecvData(const char* src)
{
    char buf[201];
    const char *p = src;
    int len = strlen(src);
    int nokori = len;
    
    while(nokori > 0) {
        int i = 200;
        if(i > nokori) i = nokori;
        strncpy(buf, p, i); buf[i] = 0;
        OSYSPRINT((buf));
        nokori -= i;
        p += i;
    }
}
